﻿/**
 * Copyright Amazon.com, Inc. or its affiliates. All Rights Reserved.
 * SPDX-License-Identifier: Apache-2.0.
 */

#pragma once
#include <aws/supplychain/SupplyChain_EXPORTS.h>
#include <aws/core/client/ClientConfiguration.h>
#include <aws/core/client/AWSClient.h>
#include <aws/core/client/AWSClientAsyncCRTP.h>
#include <aws/core/utils/json/JsonSerializer.h>
#include <aws/supplychain/SupplyChainServiceClientModel.h>

namespace Aws
{
namespace SupplyChain
{
  /**
   * <p> AWS Supply Chain is a cloud-based application that works with your
   * enterprise resource planning (ERP) and supply chain management systems. Using
   * AWS Supply Chain, you can connect and extract your inventory, supply, and demand
   * related data from existing ERP or supply chain systems into a single data model.
   * </p> <p>The AWS Supply Chain API supports configuration data import for Supply
   * Planning.</p> <p> All AWS Supply chain API operations are Amazon-authenticated
   * and certificate-signed. They not only require the use of the AWS SDK, but also
   * allow for the exclusive use of AWS Identity and Access Management users and
   * roles to help facilitate access, trust, and permission policies. </p>
   */
  class AWS_SUPPLYCHAIN_API SupplyChainClient : public Aws::Client::AWSJsonClient, public Aws::Client::ClientWithAsyncTemplateMethods<SupplyChainClient>
  {
    public:
      typedef Aws::Client::AWSJsonClient BASECLASS;
      static const char* GetServiceName();
      static const char* GetAllocationTag();

      typedef SupplyChainClientConfiguration ClientConfigurationType;
      typedef SupplyChainEndpointProvider EndpointProviderType;

       /**
        * Initializes client to use DefaultCredentialProviderChain, with default http client factory, and optional client config. If client config
        * is not specified, it will be initialized to default values.
        */
        SupplyChainClient(const Aws::SupplyChain::SupplyChainClientConfiguration& clientConfiguration = Aws::SupplyChain::SupplyChainClientConfiguration(),
                          std::shared_ptr<SupplyChainEndpointProviderBase> endpointProvider = nullptr);

       /**
        * Initializes client to use SimpleAWSCredentialsProvider, with default http client factory, and optional client config. If client config
        * is not specified, it will be initialized to default values.
        */
        SupplyChainClient(const Aws::Auth::AWSCredentials& credentials,
                          std::shared_ptr<SupplyChainEndpointProviderBase> endpointProvider = nullptr,
                          const Aws::SupplyChain::SupplyChainClientConfiguration& clientConfiguration = Aws::SupplyChain::SupplyChainClientConfiguration());

       /**
        * Initializes client to use specified credentials provider with specified client config. If http client factory is not supplied,
        * the default http client factory will be used
        */
        SupplyChainClient(const std::shared_ptr<Aws::Auth::AWSCredentialsProvider>& credentialsProvider,
                          std::shared_ptr<SupplyChainEndpointProviderBase> endpointProvider = nullptr,
                          const Aws::SupplyChain::SupplyChainClientConfiguration& clientConfiguration = Aws::SupplyChain::SupplyChainClientConfiguration());


        /* Legacy constructors due deprecation */
       /**
        * Initializes client to use DefaultCredentialProviderChain, with default http client factory, and optional client config. If client config
        * is not specified, it will be initialized to default values.
        */
        SupplyChainClient(const Aws::Client::ClientConfiguration& clientConfiguration);

       /**
        * Initializes client to use SimpleAWSCredentialsProvider, with default http client factory, and optional client config. If client config
        * is not specified, it will be initialized to default values.
        */
        SupplyChainClient(const Aws::Auth::AWSCredentials& credentials,
                          const Aws::Client::ClientConfiguration& clientConfiguration);

       /**
        * Initializes client to use specified credentials provider with specified client config. If http client factory is not supplied,
        * the default http client factory will be used
        */
        SupplyChainClient(const std::shared_ptr<Aws::Auth::AWSCredentialsProvider>& credentialsProvider,
                          const Aws::Client::ClientConfiguration& clientConfiguration);

        /* End of legacy constructors due deprecation */
        virtual ~SupplyChainClient();

        /**
         * <p>CreateBillOfMaterialsImportJob creates an import job for the Product Bill Of
         * Materials (BOM) entity. For information on the product_bom entity, see the AWS
         * Supply Chain User Guide.</p> <p>The CSV file must be located in an Amazon S3
         * location accessible to AWS Supply Chain. It is recommended to use the same
         * Amazon S3 bucket created during your AWS Supply Chain instance
         * creation.</p><p><h3>See Also:</h3>   <a
         * href="http://docs.aws.amazon.com/goto/WebAPI/supplychain-2024-01-01/CreateBillOfMaterialsImportJob">AWS
         * API Reference</a></p>
         */
        virtual Model::CreateBillOfMaterialsImportJobOutcome CreateBillOfMaterialsImportJob(const Model::CreateBillOfMaterialsImportJobRequest& request) const;

        /**
         * A Callable wrapper for CreateBillOfMaterialsImportJob that returns a future to the operation so that it can be executed in parallel to other requests.
         */
        template<typename CreateBillOfMaterialsImportJobRequestT = Model::CreateBillOfMaterialsImportJobRequest>
        Model::CreateBillOfMaterialsImportJobOutcomeCallable CreateBillOfMaterialsImportJobCallable(const CreateBillOfMaterialsImportJobRequestT& request) const
        {
            return SubmitCallable(&SupplyChainClient::CreateBillOfMaterialsImportJob, request);
        }

        /**
         * An Async wrapper for CreateBillOfMaterialsImportJob that queues the request into a thread executor and triggers associated callback when operation has finished.
         */
        template<typename CreateBillOfMaterialsImportJobRequestT = Model::CreateBillOfMaterialsImportJobRequest>
        void CreateBillOfMaterialsImportJobAsync(const CreateBillOfMaterialsImportJobRequestT& request, const CreateBillOfMaterialsImportJobResponseReceivedHandler& handler, const std::shared_ptr<const Aws::Client::AsyncCallerContext>& context = nullptr) const
        {
            return SubmitAsync(&SupplyChainClient::CreateBillOfMaterialsImportJob, request, handler, context);
        }

        /**
         * <p>Create DataIntegrationFlow to map one or more different sources to one target
         * using the SQL transformation query.</p><p><h3>See Also:</h3>   <a
         * href="http://docs.aws.amazon.com/goto/WebAPI/supplychain-2024-01-01/CreateDataIntegrationFlow">AWS
         * API Reference</a></p>
         */
        virtual Model::CreateDataIntegrationFlowOutcome CreateDataIntegrationFlow(const Model::CreateDataIntegrationFlowRequest& request) const;

        /**
         * A Callable wrapper for CreateDataIntegrationFlow that returns a future to the operation so that it can be executed in parallel to other requests.
         */
        template<typename CreateDataIntegrationFlowRequestT = Model::CreateDataIntegrationFlowRequest>
        Model::CreateDataIntegrationFlowOutcomeCallable CreateDataIntegrationFlowCallable(const CreateDataIntegrationFlowRequestT& request) const
        {
            return SubmitCallable(&SupplyChainClient::CreateDataIntegrationFlow, request);
        }

        /**
         * An Async wrapper for CreateDataIntegrationFlow that queues the request into a thread executor and triggers associated callback when operation has finished.
         */
        template<typename CreateDataIntegrationFlowRequestT = Model::CreateDataIntegrationFlowRequest>
        void CreateDataIntegrationFlowAsync(const CreateDataIntegrationFlowRequestT& request, const CreateDataIntegrationFlowResponseReceivedHandler& handler, const std::shared_ptr<const Aws::Client::AsyncCallerContext>& context = nullptr) const
        {
            return SubmitAsync(&SupplyChainClient::CreateDataIntegrationFlow, request, handler, context);
        }

        /**
         * <p>Create a data lake dataset.</p><p><h3>See Also:</h3>   <a
         * href="http://docs.aws.amazon.com/goto/WebAPI/supplychain-2024-01-01/CreateDataLakeDataset">AWS
         * API Reference</a></p>
         */
        virtual Model::CreateDataLakeDatasetOutcome CreateDataLakeDataset(const Model::CreateDataLakeDatasetRequest& request) const;

        /**
         * A Callable wrapper for CreateDataLakeDataset that returns a future to the operation so that it can be executed in parallel to other requests.
         */
        template<typename CreateDataLakeDatasetRequestT = Model::CreateDataLakeDatasetRequest>
        Model::CreateDataLakeDatasetOutcomeCallable CreateDataLakeDatasetCallable(const CreateDataLakeDatasetRequestT& request) const
        {
            return SubmitCallable(&SupplyChainClient::CreateDataLakeDataset, request);
        }

        /**
         * An Async wrapper for CreateDataLakeDataset that queues the request into a thread executor and triggers associated callback when operation has finished.
         */
        template<typename CreateDataLakeDatasetRequestT = Model::CreateDataLakeDatasetRequest>
        void CreateDataLakeDatasetAsync(const CreateDataLakeDatasetRequestT& request, const CreateDataLakeDatasetResponseReceivedHandler& handler, const std::shared_ptr<const Aws::Client::AsyncCallerContext>& context = nullptr) const
        {
            return SubmitAsync(&SupplyChainClient::CreateDataLakeDataset, request, handler, context);
        }

        /**
         * <p>Delete the DataIntegrationFlow.</p><p><h3>See Also:</h3>   <a
         * href="http://docs.aws.amazon.com/goto/WebAPI/supplychain-2024-01-01/DeleteDataIntegrationFlow">AWS
         * API Reference</a></p>
         */
        virtual Model::DeleteDataIntegrationFlowOutcome DeleteDataIntegrationFlow(const Model::DeleteDataIntegrationFlowRequest& request) const;

        /**
         * A Callable wrapper for DeleteDataIntegrationFlow that returns a future to the operation so that it can be executed in parallel to other requests.
         */
        template<typename DeleteDataIntegrationFlowRequestT = Model::DeleteDataIntegrationFlowRequest>
        Model::DeleteDataIntegrationFlowOutcomeCallable DeleteDataIntegrationFlowCallable(const DeleteDataIntegrationFlowRequestT& request) const
        {
            return SubmitCallable(&SupplyChainClient::DeleteDataIntegrationFlow, request);
        }

        /**
         * An Async wrapper for DeleteDataIntegrationFlow that queues the request into a thread executor and triggers associated callback when operation has finished.
         */
        template<typename DeleteDataIntegrationFlowRequestT = Model::DeleteDataIntegrationFlowRequest>
        void DeleteDataIntegrationFlowAsync(const DeleteDataIntegrationFlowRequestT& request, const DeleteDataIntegrationFlowResponseReceivedHandler& handler, const std::shared_ptr<const Aws::Client::AsyncCallerContext>& context = nullptr) const
        {
            return SubmitAsync(&SupplyChainClient::DeleteDataIntegrationFlow, request, handler, context);
        }

        /**
         * <p>Delete a data lake dataset.</p><p><h3>See Also:</h3>   <a
         * href="http://docs.aws.amazon.com/goto/WebAPI/supplychain-2024-01-01/DeleteDataLakeDataset">AWS
         * API Reference</a></p>
         */
        virtual Model::DeleteDataLakeDatasetOutcome DeleteDataLakeDataset(const Model::DeleteDataLakeDatasetRequest& request) const;

        /**
         * A Callable wrapper for DeleteDataLakeDataset that returns a future to the operation so that it can be executed in parallel to other requests.
         */
        template<typename DeleteDataLakeDatasetRequestT = Model::DeleteDataLakeDatasetRequest>
        Model::DeleteDataLakeDatasetOutcomeCallable DeleteDataLakeDatasetCallable(const DeleteDataLakeDatasetRequestT& request) const
        {
            return SubmitCallable(&SupplyChainClient::DeleteDataLakeDataset, request);
        }

        /**
         * An Async wrapper for DeleteDataLakeDataset that queues the request into a thread executor and triggers associated callback when operation has finished.
         */
        template<typename DeleteDataLakeDatasetRequestT = Model::DeleteDataLakeDatasetRequest>
        void DeleteDataLakeDatasetAsync(const DeleteDataLakeDatasetRequestT& request, const DeleteDataLakeDatasetResponseReceivedHandler& handler, const std::shared_ptr<const Aws::Client::AsyncCallerContext>& context = nullptr) const
        {
            return SubmitAsync(&SupplyChainClient::DeleteDataLakeDataset, request, handler, context);
        }

        /**
         * <p>Get status and details of a BillOfMaterialsImportJob.</p><p><h3>See
         * Also:</h3>   <a
         * href="http://docs.aws.amazon.com/goto/WebAPI/supplychain-2024-01-01/GetBillOfMaterialsImportJob">AWS
         * API Reference</a></p>
         */
        virtual Model::GetBillOfMaterialsImportJobOutcome GetBillOfMaterialsImportJob(const Model::GetBillOfMaterialsImportJobRequest& request) const;

        /**
         * A Callable wrapper for GetBillOfMaterialsImportJob that returns a future to the operation so that it can be executed in parallel to other requests.
         */
        template<typename GetBillOfMaterialsImportJobRequestT = Model::GetBillOfMaterialsImportJobRequest>
        Model::GetBillOfMaterialsImportJobOutcomeCallable GetBillOfMaterialsImportJobCallable(const GetBillOfMaterialsImportJobRequestT& request) const
        {
            return SubmitCallable(&SupplyChainClient::GetBillOfMaterialsImportJob, request);
        }

        /**
         * An Async wrapper for GetBillOfMaterialsImportJob that queues the request into a thread executor and triggers associated callback when operation has finished.
         */
        template<typename GetBillOfMaterialsImportJobRequestT = Model::GetBillOfMaterialsImportJobRequest>
        void GetBillOfMaterialsImportJobAsync(const GetBillOfMaterialsImportJobRequestT& request, const GetBillOfMaterialsImportJobResponseReceivedHandler& handler, const std::shared_ptr<const Aws::Client::AsyncCallerContext>& context = nullptr) const
        {
            return SubmitAsync(&SupplyChainClient::GetBillOfMaterialsImportJob, request, handler, context);
        }

        /**
         * <p>View the DataIntegrationFlow details.</p><p><h3>See Also:</h3>   <a
         * href="http://docs.aws.amazon.com/goto/WebAPI/supplychain-2024-01-01/GetDataIntegrationFlow">AWS
         * API Reference</a></p>
         */
        virtual Model::GetDataIntegrationFlowOutcome GetDataIntegrationFlow(const Model::GetDataIntegrationFlowRequest& request) const;

        /**
         * A Callable wrapper for GetDataIntegrationFlow that returns a future to the operation so that it can be executed in parallel to other requests.
         */
        template<typename GetDataIntegrationFlowRequestT = Model::GetDataIntegrationFlowRequest>
        Model::GetDataIntegrationFlowOutcomeCallable GetDataIntegrationFlowCallable(const GetDataIntegrationFlowRequestT& request) const
        {
            return SubmitCallable(&SupplyChainClient::GetDataIntegrationFlow, request);
        }

        /**
         * An Async wrapper for GetDataIntegrationFlow that queues the request into a thread executor and triggers associated callback when operation has finished.
         */
        template<typename GetDataIntegrationFlowRequestT = Model::GetDataIntegrationFlowRequest>
        void GetDataIntegrationFlowAsync(const GetDataIntegrationFlowRequestT& request, const GetDataIntegrationFlowResponseReceivedHandler& handler, const std::shared_ptr<const Aws::Client::AsyncCallerContext>& context = nullptr) const
        {
            return SubmitAsync(&SupplyChainClient::GetDataIntegrationFlow, request, handler, context);
        }

        /**
         * <p>Get a data lake dataset.</p><p><h3>See Also:</h3>   <a
         * href="http://docs.aws.amazon.com/goto/WebAPI/supplychain-2024-01-01/GetDataLakeDataset">AWS
         * API Reference</a></p>
         */
        virtual Model::GetDataLakeDatasetOutcome GetDataLakeDataset(const Model::GetDataLakeDatasetRequest& request) const;

        /**
         * A Callable wrapper for GetDataLakeDataset that returns a future to the operation so that it can be executed in parallel to other requests.
         */
        template<typename GetDataLakeDatasetRequestT = Model::GetDataLakeDatasetRequest>
        Model::GetDataLakeDatasetOutcomeCallable GetDataLakeDatasetCallable(const GetDataLakeDatasetRequestT& request) const
        {
            return SubmitCallable(&SupplyChainClient::GetDataLakeDataset, request);
        }

        /**
         * An Async wrapper for GetDataLakeDataset that queues the request into a thread executor and triggers associated callback when operation has finished.
         */
        template<typename GetDataLakeDatasetRequestT = Model::GetDataLakeDatasetRequest>
        void GetDataLakeDatasetAsync(const GetDataLakeDatasetRequestT& request, const GetDataLakeDatasetResponseReceivedHandler& handler, const std::shared_ptr<const Aws::Client::AsyncCallerContext>& context = nullptr) const
        {
            return SubmitAsync(&SupplyChainClient::GetDataLakeDataset, request, handler, context);
        }

        /**
         * <p>Lists all the DataIntegrationFlows in a paginated way.</p><p><h3>See
         * Also:</h3>   <a
         * href="http://docs.aws.amazon.com/goto/WebAPI/supplychain-2024-01-01/ListDataIntegrationFlows">AWS
         * API Reference</a></p>
         */
        virtual Model::ListDataIntegrationFlowsOutcome ListDataIntegrationFlows(const Model::ListDataIntegrationFlowsRequest& request) const;

        /**
         * A Callable wrapper for ListDataIntegrationFlows that returns a future to the operation so that it can be executed in parallel to other requests.
         */
        template<typename ListDataIntegrationFlowsRequestT = Model::ListDataIntegrationFlowsRequest>
        Model::ListDataIntegrationFlowsOutcomeCallable ListDataIntegrationFlowsCallable(const ListDataIntegrationFlowsRequestT& request) const
        {
            return SubmitCallable(&SupplyChainClient::ListDataIntegrationFlows, request);
        }

        /**
         * An Async wrapper for ListDataIntegrationFlows that queues the request into a thread executor and triggers associated callback when operation has finished.
         */
        template<typename ListDataIntegrationFlowsRequestT = Model::ListDataIntegrationFlowsRequest>
        void ListDataIntegrationFlowsAsync(const ListDataIntegrationFlowsRequestT& request, const ListDataIntegrationFlowsResponseReceivedHandler& handler, const std::shared_ptr<const Aws::Client::AsyncCallerContext>& context = nullptr) const
        {
            return SubmitAsync(&SupplyChainClient::ListDataIntegrationFlows, request, handler, context);
        }

        /**
         * <p>List the data lake datasets for a specific instance and name
         * space.</p><p><h3>See Also:</h3>   <a
         * href="http://docs.aws.amazon.com/goto/WebAPI/supplychain-2024-01-01/ListDataLakeDatasets">AWS
         * API Reference</a></p>
         */
        virtual Model::ListDataLakeDatasetsOutcome ListDataLakeDatasets(const Model::ListDataLakeDatasetsRequest& request) const;

        /**
         * A Callable wrapper for ListDataLakeDatasets that returns a future to the operation so that it can be executed in parallel to other requests.
         */
        template<typename ListDataLakeDatasetsRequestT = Model::ListDataLakeDatasetsRequest>
        Model::ListDataLakeDatasetsOutcomeCallable ListDataLakeDatasetsCallable(const ListDataLakeDatasetsRequestT& request) const
        {
            return SubmitCallable(&SupplyChainClient::ListDataLakeDatasets, request);
        }

        /**
         * An Async wrapper for ListDataLakeDatasets that queues the request into a thread executor and triggers associated callback when operation has finished.
         */
        template<typename ListDataLakeDatasetsRequestT = Model::ListDataLakeDatasetsRequest>
        void ListDataLakeDatasetsAsync(const ListDataLakeDatasetsRequestT& request, const ListDataLakeDatasetsResponseReceivedHandler& handler, const std::shared_ptr<const Aws::Client::AsyncCallerContext>& context = nullptr) const
        {
            return SubmitAsync(&SupplyChainClient::ListDataLakeDatasets, request, handler, context);
        }

        /**
         * <p>List all the tags for an Amazon Web ServicesSupply Chain
         * resource.</p><p><h3>See Also:</h3>   <a
         * href="http://docs.aws.amazon.com/goto/WebAPI/supplychain-2024-01-01/ListTagsForResource">AWS
         * API Reference</a></p>
         */
        virtual Model::ListTagsForResourceOutcome ListTagsForResource(const Model::ListTagsForResourceRequest& request) const;

        /**
         * A Callable wrapper for ListTagsForResource that returns a future to the operation so that it can be executed in parallel to other requests.
         */
        template<typename ListTagsForResourceRequestT = Model::ListTagsForResourceRequest>
        Model::ListTagsForResourceOutcomeCallable ListTagsForResourceCallable(const ListTagsForResourceRequestT& request) const
        {
            return SubmitCallable(&SupplyChainClient::ListTagsForResource, request);
        }

        /**
         * An Async wrapper for ListTagsForResource that queues the request into a thread executor and triggers associated callback when operation has finished.
         */
        template<typename ListTagsForResourceRequestT = Model::ListTagsForResourceRequest>
        void ListTagsForResourceAsync(const ListTagsForResourceRequestT& request, const ListTagsForResourceResponseReceivedHandler& handler, const std::shared_ptr<const Aws::Client::AsyncCallerContext>& context = nullptr) const
        {
            return SubmitAsync(&SupplyChainClient::ListTagsForResource, request, handler, context);
        }

        /**
         * <p>Send the transactional data payload for the event with real-time data for
         * analysis or monitoring. The real-time data events are stored in an Amazon Web
         * Services service before being processed and stored in data lake. New data events
         * are synced with data lake at 5 PM GMT everyday. The updated transactional data
         * is available in data lake after ingestion.</p><p><h3>See Also:</h3>   <a
         * href="http://docs.aws.amazon.com/goto/WebAPI/supplychain-2024-01-01/SendDataIntegrationEvent">AWS
         * API Reference</a></p>
         */
        virtual Model::SendDataIntegrationEventOutcome SendDataIntegrationEvent(const Model::SendDataIntegrationEventRequest& request) const;

        /**
         * A Callable wrapper for SendDataIntegrationEvent that returns a future to the operation so that it can be executed in parallel to other requests.
         */
        template<typename SendDataIntegrationEventRequestT = Model::SendDataIntegrationEventRequest>
        Model::SendDataIntegrationEventOutcomeCallable SendDataIntegrationEventCallable(const SendDataIntegrationEventRequestT& request) const
        {
            return SubmitCallable(&SupplyChainClient::SendDataIntegrationEvent, request);
        }

        /**
         * An Async wrapper for SendDataIntegrationEvent that queues the request into a thread executor and triggers associated callback when operation has finished.
         */
        template<typename SendDataIntegrationEventRequestT = Model::SendDataIntegrationEventRequest>
        void SendDataIntegrationEventAsync(const SendDataIntegrationEventRequestT& request, const SendDataIntegrationEventResponseReceivedHandler& handler, const std::shared_ptr<const Aws::Client::AsyncCallerContext>& context = nullptr) const
        {
            return SubmitAsync(&SupplyChainClient::SendDataIntegrationEvent, request, handler, context);
        }

        /**
         * <p>Create tags for an Amazon Web Services Supply chain resource.</p><p><h3>See
         * Also:</h3>   <a
         * href="http://docs.aws.amazon.com/goto/WebAPI/supplychain-2024-01-01/TagResource">AWS
         * API Reference</a></p>
         */
        virtual Model::TagResourceOutcome TagResource(const Model::TagResourceRequest& request) const;

        /**
         * A Callable wrapper for TagResource that returns a future to the operation so that it can be executed in parallel to other requests.
         */
        template<typename TagResourceRequestT = Model::TagResourceRequest>
        Model::TagResourceOutcomeCallable TagResourceCallable(const TagResourceRequestT& request) const
        {
            return SubmitCallable(&SupplyChainClient::TagResource, request);
        }

        /**
         * An Async wrapper for TagResource that queues the request into a thread executor and triggers associated callback when operation has finished.
         */
        template<typename TagResourceRequestT = Model::TagResourceRequest>
        void TagResourceAsync(const TagResourceRequestT& request, const TagResourceResponseReceivedHandler& handler, const std::shared_ptr<const Aws::Client::AsyncCallerContext>& context = nullptr) const
        {
            return SubmitAsync(&SupplyChainClient::TagResource, request, handler, context);
        }

        /**
         * <p>Delete tags for an Amazon Web Services Supply chain resource.</p><p><h3>See
         * Also:</h3>   <a
         * href="http://docs.aws.amazon.com/goto/WebAPI/supplychain-2024-01-01/UntagResource">AWS
         * API Reference</a></p>
         */
        virtual Model::UntagResourceOutcome UntagResource(const Model::UntagResourceRequest& request) const;

        /**
         * A Callable wrapper for UntagResource that returns a future to the operation so that it can be executed in parallel to other requests.
         */
        template<typename UntagResourceRequestT = Model::UntagResourceRequest>
        Model::UntagResourceOutcomeCallable UntagResourceCallable(const UntagResourceRequestT& request) const
        {
            return SubmitCallable(&SupplyChainClient::UntagResource, request);
        }

        /**
         * An Async wrapper for UntagResource that queues the request into a thread executor and triggers associated callback when operation has finished.
         */
        template<typename UntagResourceRequestT = Model::UntagResourceRequest>
        void UntagResourceAsync(const UntagResourceRequestT& request, const UntagResourceResponseReceivedHandler& handler, const std::shared_ptr<const Aws::Client::AsyncCallerContext>& context = nullptr) const
        {
            return SubmitAsync(&SupplyChainClient::UntagResource, request, handler, context);
        }

        /**
         * <p>Update the DataIntegrationFlow.</p><p><h3>See Also:</h3>   <a
         * href="http://docs.aws.amazon.com/goto/WebAPI/supplychain-2024-01-01/UpdateDataIntegrationFlow">AWS
         * API Reference</a></p>
         */
        virtual Model::UpdateDataIntegrationFlowOutcome UpdateDataIntegrationFlow(const Model::UpdateDataIntegrationFlowRequest& request) const;

        /**
         * A Callable wrapper for UpdateDataIntegrationFlow that returns a future to the operation so that it can be executed in parallel to other requests.
         */
        template<typename UpdateDataIntegrationFlowRequestT = Model::UpdateDataIntegrationFlowRequest>
        Model::UpdateDataIntegrationFlowOutcomeCallable UpdateDataIntegrationFlowCallable(const UpdateDataIntegrationFlowRequestT& request) const
        {
            return SubmitCallable(&SupplyChainClient::UpdateDataIntegrationFlow, request);
        }

        /**
         * An Async wrapper for UpdateDataIntegrationFlow that queues the request into a thread executor and triggers associated callback when operation has finished.
         */
        template<typename UpdateDataIntegrationFlowRequestT = Model::UpdateDataIntegrationFlowRequest>
        void UpdateDataIntegrationFlowAsync(const UpdateDataIntegrationFlowRequestT& request, const UpdateDataIntegrationFlowResponseReceivedHandler& handler, const std::shared_ptr<const Aws::Client::AsyncCallerContext>& context = nullptr) const
        {
            return SubmitAsync(&SupplyChainClient::UpdateDataIntegrationFlow, request, handler, context);
        }

        /**
         * <p>Update a data lake dataset.</p><p><h3>See Also:</h3>   <a
         * href="http://docs.aws.amazon.com/goto/WebAPI/supplychain-2024-01-01/UpdateDataLakeDataset">AWS
         * API Reference</a></p>
         */
        virtual Model::UpdateDataLakeDatasetOutcome UpdateDataLakeDataset(const Model::UpdateDataLakeDatasetRequest& request) const;

        /**
         * A Callable wrapper for UpdateDataLakeDataset that returns a future to the operation so that it can be executed in parallel to other requests.
         */
        template<typename UpdateDataLakeDatasetRequestT = Model::UpdateDataLakeDatasetRequest>
        Model::UpdateDataLakeDatasetOutcomeCallable UpdateDataLakeDatasetCallable(const UpdateDataLakeDatasetRequestT& request) const
        {
            return SubmitCallable(&SupplyChainClient::UpdateDataLakeDataset, request);
        }

        /**
         * An Async wrapper for UpdateDataLakeDataset that queues the request into a thread executor and triggers associated callback when operation has finished.
         */
        template<typename UpdateDataLakeDatasetRequestT = Model::UpdateDataLakeDatasetRequest>
        void UpdateDataLakeDatasetAsync(const UpdateDataLakeDatasetRequestT& request, const UpdateDataLakeDatasetResponseReceivedHandler& handler, const std::shared_ptr<const Aws::Client::AsyncCallerContext>& context = nullptr) const
        {
            return SubmitAsync(&SupplyChainClient::UpdateDataLakeDataset, request, handler, context);
        }


      void OverrideEndpoint(const Aws::String& endpoint);
      std::shared_ptr<SupplyChainEndpointProviderBase>& accessEndpointProvider();
    private:
      friend class Aws::Client::ClientWithAsyncTemplateMethods<SupplyChainClient>;
      void init(const SupplyChainClientConfiguration& clientConfiguration);

      SupplyChainClientConfiguration m_clientConfiguration;
      std::shared_ptr<SupplyChainEndpointProviderBase> m_endpointProvider;
  };

} // namespace SupplyChain
} // namespace Aws
